package bag

import (
	"f5"
	"main/common"
	"main/constant"
	"main/model"
	"main/mt"
	"main/service"
	"main/vo"
	"q5"

	"github.com/gin-gonic/gin"
)

type BagApi struct {
}

func (this *BagApi) List(c *gin.Context) {
	s := c.MustGet(constant.SESSION_KEY).(common.Session)
	if s == nil {
		return
	}
	err, items := service.Bag.List(s.GetAccountId())
	if err != nil {
		f5.RspErr(c, 500, "server internal error")
		return
	}
	rspObj := struct {
		vo.BaseVo
		Data []*vo.BagItem `json:"data"`
	}{
		Data: []*vo.BagItem{},
	}
	for _, m := range items {
		v := new(vo.BagItem)
		v.FromModel(m)
		q5.AppendSlice(&rspObj.Data, v)
	}
	c.JSON(200, rspObj)
}

func (this *BagApi) UseItem(c *gin.Context) {
	s := c.MustGet(constant.SESSION_KEY).(common.Session)
	if s == nil {
		return
	}
	reqJson := struct {
		ItemUniId string `json:"item_uniid"`
		ItemNum int32 `json:"item_num"`
	}{}
	if err := c.ShouldBindJSON(&reqJson); err != nil {
		f5.RspErr(c, 401, "params parse error")
		return
	}
	rspObj := struct {
		vo.BaseVo
	}{}
	bagItem := new(model.Bag)
	if err, found := bagItem.FindByItemUniId(s.GetAccountId(), q5.ToInt64(reqJson.ItemUniId)); err != nil  {
		f5.RspErr(c, 500, "server internal error")
		return
	} else if !found {
		f5.RspErr(c, 1, "item not found")
		return
	}
	if bagItem.ItemNum < reqJson.ItemNum {
		f5.RspErr(c, 1, "item not enough")
		return
	}
	nowTime := f5.GetApp().GetRealSeconds()
	itemMeta := mt.Table.Item.GetById(int64(bagItem.ItemId))
	if itemMeta == nil {
		f5.RspErr(c, 500, "server internal error")
		return
	}
	if itemMeta.GetItemType() != constant.ITEM_TYPE_SCORE_DOUBLE_CARD {
		f5.RspErr(c, 500, "cant use")
		return
	}
	service.Buff.Add(s.GetAccountId(), itemMeta.GetBuffList())
	bagItem.DecItemNum(1, int32(nowTime))
	q5.AppendSlice(&rspObj.BaseVo.GetOrCreateSideEffect().Effects, "buff")
	q5.AppendSlice(&rspObj.BaseVo.GetOrCreateSideEffect().Effects, "bag")
	if service.Task.UpdateTaskProgress(s.GetAccountId(), itemMeta.GetItemType(), reqJson.ItemNum, nil) {
		q5.AppendSlice(&rspObj.BaseVo.GetOrCreateSideEffect().Effects, "task")
	}
	c.JSON(200, rspObj)
}
