package user

import (
	"f5"
	"main/common"
	"main/constant"
	"main/model"
	"main/service"
	"main/vo"
	"q5"

	"github.com/gin-gonic/gin"
)

type UserApi struct {
}

func (this *UserApi) Login(c *gin.Context) {
	s := c.MustGet(constant.SESSION_KEY).(common.Session)
	if s == nil {
		return
	}
	user := new(model.User)
	rspObj := struct {
		vo.BaseVo
		UserInfo   vo.User `json:"user_info"`
		ServerTime int32   `json:"server_time"`
	}{}
	nowTime := f5.GetApp().GetRealSeconds()
	rspObj.ServerTime = int32(nowTime)
	err, found := user.Find(s.GetAccountId(), nowTime)
	if err != nil {
		f5.RspErr(c, 500, "server internal error")
		return
	} else if !found {
		user.AccountId = s.GetAccountId()
		user.Uid = s.GetUid()
		user.Gid = s.GetGid()
		user.OpenId = s.GetOpenId()
		user.Invited = s.GetInvited()
		user.Avatar = ""
		user.NickName = s.GetNickName()
		user.Score = 0
		user.Dice = constant.PRESENT_DICE_LIMIT//mt.Table.Global.GetDailyDiceNum()
		user.LastPresentDiceTime = q5.ToInt32(nowTime)
		user.CurrGrid = 1
		user.CreateTime = q5.ToInt32(nowTime)
		user.ModifyTime = q5.ToInt32(nowTime)
		if user.Create() != nil {
			f5.RspErr(c, 500, "server internal error")
			return
		}
		f5.GetMsgQueue().FireEvent(constant.MSG_CREATE_USER, q5.Args{user})
		service.Task.List(s.GetAccountId(), nil)
	} else {
		earning := service.Chip.CalcScore(s.GetAccountId())
		if earning > 0 {
			user.AddScore(earning)
			if service.Task.UpdateTaskProgress(s.GetAccountId(), constant.ITEM_TYPE_SCORE, earning, nil) {
				q5.AppendSlice(&rspObj.BaseVo.GetOrCreateSideEffect().Effects, "task")
			}			
		}
	}

	if user.NickName != s.GetNickName() {
		user.UpdateName()
	}
	f5.GetMsgQueue().FireEvent(constant.MSG_LOGIN, q5.Args{user})
	rspObj.UserInfo.FromModel(user)
	if found {
		rspObj.UserInfo.HourlyEarnings = service.Chip.GetHourEarning(user.AccountId)
	}
	c.JSON(200, rspObj)
}

func (this *UserApi) Info(c *gin.Context) {
	s := c.MustGet(constant.SESSION_KEY).(common.Session)
	if s == nil {
		return
	}
	user := new(model.User)
	rspObj := struct {
		vo.BaseVo
		UserInfo vo.User `json:"user_info"`
	}{}
	nowTime := f5.GetApp().GetRealSeconds()
	if err, found := user.Find(s.GetAccountId(), nowTime); err != nil {
		f5.RspErr(c, 500, "server internal error")
		return
	} else if !found {
		f5.RspErr(c, 1, "server internal error")
		return
	}
	rspObj.UserInfo.FromModel(user)
	rspObj.UserInfo.HourlyEarnings = service.Chip.GetHourEarning(user.AccountId)
	c.JSON(200, rspObj)
}